@@ -25,6 +25,7 @@
     \file dvframe.cc
 */
 
+#include <string.h>
 #include "dvframe.h"
 
 VideoInfo::VideoInfo() : width( 0 ), height( 0 ), isPAL( false )
@@ -457,12 +458,19 @@ bool DVFrame::GetTimeCode( TimeCode &timeCode )
 #ifdef HAVE_LIBDV
 	int timestamp[ 4 ];
 
-	dv_get_timestamp_int( decoder, timestamp );
-
-	timeCode.hour = timestamp[ 0 ];
-	timeCode.min = timestamp[ 1 ];
-	timeCode.sec = timestamp[ 2 ];
-	timeCode.frame = timestamp[ 3 ];
+	if ( dv_get_timestamp_int( decoder, timestamp ) ) {
+		timeCode.hour = timestamp[ 0 ];
+		timeCode.min = timestamp[ 1 ];
+		timeCode.sec = timestamp[ 2 ];
+		timeCode.frame = timestamp[ 3 ];
+		return true;
+	} else {
+		timeCode.hour = 0;
+		timeCode.min = 0;
+		timeCode.sec = 0;
+		timeCode.frame = 0;
+		return false;
+	}
 #else
 
 	Pack tc;
@@ -479,9 +487,8 @@ bool DVFrame::GetTimeCode( TimeCode &timeCode )
 	timeCode.sec = ( sec & 0xf ) + 10 * ( ( sec >> 4 ) & 0x7 );
 	timeCode.min = ( min & 0xf ) + 10 * ( ( min >> 4 ) & 0x7 );
 	timeCode.hour = ( hour & 0xf ) + 10 * ( ( hour >> 4 ) & 0x3 );
-#endif
-
 	return true;
+#endif
 }
 
 
