@@ -36,6 +36,7 @@ using std::endl;
 #include <stdlib.h>
 #include <stdarg.h>
 #include <time.h>
+#include <string.h>
 #include <sys/select.h>
 #include <libavc1394/avc1394.h>
 #include <libavc1394/avc1394_vcr.h>
@@ -49,6 +50,7 @@ using std::endl;
 #include "smiltime.h"
 #include "stringutils.h"
 #include "v4l2reader.h"
+#include "srt.h"
 
 extern bool g_done;
 pthread_mutex_t DVgrab::capture_mutex;
@@ -56,6 +58,8 @@ pthread_t DVgrab::capture_thread;
 pthread_t DVgrab::watchdog_thread;
 Frame *DVgrab::m_frame;
 FileHandler *DVgrab::m_writer;
+static SubtitleWriter subWriter;
+
 
 DVgrab::DVgrab( int argc, char *argv[] ) :
 		m_program_name( argv[0] ), m_port( -1 ), m_node( -1 ), m_reader_active( false ), m_autosplit( false ),
@@ -72,7 +76,8 @@ DVgrab::DVgrab( int argc, char *argv[] ) :
 		m_lockstep_maxdrops( DEFAULT_LOCKSTEP_MAXDROPS ), m_lockstep_totaldrops( DEFAULT_LOCKSTEP_TOTALDROPS ),
 		m_captureActive( false ), m_avc( 0 ), m_reader( 0 ), m_hdv( false ), m_showstatus( false ),
 		m_isLastTimeCodeSet( false ), m_isLastRecDateSet( false ), m_v4l2( false ), m_jvc_p25( false ),
-		m_24p( false ), m_24pa( false )
+		m_24p( false ), m_24pa( false ), m_isRecordMode( false ), m_isRewindFirst( false ),
+		m_timeSplit(0), m_srt( false ), m_isNewFile(false)
 {
 	m_frame = 0;
 	m_writer = 0;
@@ -169,7 +174,8 @@ void DVgrab::print_help()
 	cerr << "Usage: " << m_program_name << " [options] [file] [-]" << endl << endl;
 	cerr << "Use '-' to pipe video to stdout." << endl << endl;
 	cerr << "Options:" << endl << endl;
-	cerr << "  -a, -autosplit        start a new file when a new recording is detected" << endl;
+	cerr << "  -a[n],-autosplit[=n] start a new file when a new recording is detected" << endl;
+	cerr << "                         or if n is supplied, after n seconds gap in recording date/time" << endl;
 	cerr << "  -buffers number      the number of internal frames to buffer [default " << DEFAULT_BUFFERS << "]" << endl;
 	cerr << "  -card number         card number [default automatic]" << endl;
 	cerr << "  -channel number      iso channel number for listening [default " << DEFAULT_CHANNEL << "]" << endl;
@@ -183,7 +189,7 @@ void DVgrab::print_help()
 	cerr << "                          [[HH:]MM:]SS[.ms], or smpte=[[[HH:]MM:]SS:]FF" << endl;
 	cerr << "                          [default unlimited]" << endl;
 	cerr << "  -every number        write every n'th frame only [default " << DEFAULT_EVERY << "]" << endl;
-	cerr << "  -f -format [type]    save as one of the following file types [default " << DEFAULT_FORMAT_STR << "]" << endl;
+	cerr << "  -f -format type      save as one of the following file types [default " << DEFAULT_FORMAT_STR << "]" << endl;
 	cerr << "              raw         raw DV file with a .dv extension" << endl;
 	cerr << "              dif         raw DV file with a .dif extension" << endl;
 	cerr << "              dv1         'Type 1' DV AVI file" << endl;
@@ -221,8 +227,11 @@ void DVgrab::print_help()
 	cerr << "  -nostop              do not send AV/C stop command on exit" << endl;
 	cerr << "  -opendml             use the OpenDML extensions to write large (>1GB)" << endl;
 	cerr << "                          'Type 2' DV AVI files (requires -format dv2)" << endl;
+	cerr << "  -r, recordonly       only capture when not paused while in record mode" << endl;
+	cerr << "  -rewind              completely rewind the tape prior to capture" << endl;
 	cerr << "  -showstatus          show the recording status while capturing" << endl;
 	cerr << "  -s, -size number     max file size, 0 = unlimited [default " << DEFAULT_SIZE << "]" << endl;
+	cerr << "  -srt                 write SRT files with the recording date\n";
 	cerr << "  -stdin               read from stdin pipe [default = raw1394]" << endl;
 	cerr << "  -timecode            put the first frame's timecode into the file name" << endl;
 	cerr << "  -t, -timestamp       put the date and time of recording into the file name" << endl;
@@ -299,12 +308,12 @@ void DVgrab::set_format_from_name( void )
 
 void DVgrab::getargs( int argc, char *argv[] )
 {
-	const char *opts = "d:hif:F:I:s:tVv-";
+	const char *opts = "a::d:hif:F:I:rs:tVv-";
 	int optindex = 0;
 	int c;
 	struct option long_opts[] = {
 		// all these use sscanf for int conversion, use val == 0xff to indicate
-		{ "autosplit", no_argument, &m_autosplit, true },
+		{ "autosplit", optional_argument, 0, 'a' },
 		{ "buffers", required_argument, &m_buffers, 0xff },
 		{ "card", required_argument, &m_port, 0xff },
 		{ "channel", required_argument, &m_channel, 0xff },
@@ -334,8 +343,11 @@ void DVgrab::getargs( int argc, char *argv[] )
 		{ "noavc", no_argument, &m_noavc, true },
 		{ "nostop", no_argument, &m_no_stop, true },
 		{ "opendml", no_argument, &m_open_dml, true },
+		{ "recordonly", no_argument, 0, 'r'},
+		{ "rewind", no_argument, &m_isRewindFirst, true },
 		{ "showstatus", no_argument, &m_showstatus, true },
 		{ "size", required_argument, &m_max_file_size, 0xff },
+		{ "srt", no_argument, &m_srt, true },
 		{ "stdin", no_argument, 0, 0 },
 		{ "timecode", no_argument, &m_timecode, true },
 		{ "timestamp", no_argument, &m_timestamp, true },
@@ -417,6 +429,12 @@ void DVgrab::getargs( int argc, char *argv[] )
 					m_duration = optarg;
 			}
 			break;
+		case 'a':
+			if ( optarg )
+				m_timeSplit = atoi( optarg );
+			else
+				m_autosplit = true;
+			break;
 		case 'd':
 			m_duration = optarg;
 			break;
@@ -436,6 +454,9 @@ void DVgrab::getargs( int argc, char *argv[] )
 		case 'I':
 			m_input_file_name = optarg;
 			break;
+		case 'r':
+			m_isRecordMode = true;
+			break;
 		case 's':
 			m_max_file_size = atoi( optarg );
 			break;
@@ -465,7 +486,7 @@ void DVgrab::getargs( int argc, char *argv[] )
 	}
 	if ( optind < argc )
 	{
-		m_dst_file_name = argv[ optind++ ];
+		m_dst_file_name = strdup( argv[ optind++ ] );
 		set_format_from_name();
 	}
 	if ( optind < argc )
@@ -549,6 +570,7 @@ void DVgrab::startCapture()
 		m_writer->SetBaseName( m_dst_file_name );
 		m_writer->SetMaxFrameCount( m_frame_count );
 		m_writer->SetAutoSplit( m_autosplit );
+		m_writer->SetTimeSplit ( m_timeSplit );
 		m_writer->SetEveryNthFrame( m_frame_every );
 		m_writer->SetMaxFileSize( ( off_t ) m_max_file_size * ( off_t ) ( 1024 * 1024 ) );
 		if (m_collection_size) {
@@ -557,11 +579,44 @@ void DVgrab::startCapture()
 		m_writer->SetSizeSplitMode( m_sizesplitmode );
 		m_writer->SetMaxColSize( ( off_t ) ( m_collection_size ) * ( off_t ) ( 1024 * 1024 ) );
 		m_writer->SetMinColSize( ( off_t ) ( m_collection_size - m_collection_min_cut_file_size ) * ( off_t ) ( 1024 * 1024 ) );
-		
 	}
-	if ( m_avc )
-		m_avc->Play( m_node );
 
+	if ( m_avc )
+	{
+		if ( m_isRewindFirst && !m_interactive )
+		{
+			// Stop whatever is happening
+			m_avc->Stop( m_node );
+
+			// Wait until it is stopped
+			while ( !g_done &&
+				AVC1394_MASK_RESPONSE_OPERAND( m_avc->TransportStatus( m_node ), 3 )
+				!= AVC1394_VCR_OPERAND_WIND_STOP )
+			{
+				timespec t = {0, 25000000UL};
+				nanosleep( &t, NULL );
+			}
+
+			// Rewind
+			if ( !g_done )
+				m_avc->Rewind( m_node );
+
+			// Wait until is done rewinding
+			while ( !g_done &&
+			        AVC1394_MASK_RESPONSE_OPERAND( m_avc->TransportStatus( m_node ), 3 )
+			            != AVC1394_VCR_OPERAND_WIND_STOP )
+			{
+				timespec t = {0, 25000000UL};
+				nanosleep( &t, NULL );
+			}
+		}
+
+		// Now Play so we can capture something
+		if ( !g_done )
+			m_avc->Play( m_node );
+	}
+
+	// Wait up to N seconds to see if we got any data
 	// this is a little unclean, checking global g_done from main.cc to allow interruption
 	while ( !g_done && m_frame == NULL && tryCounter-- > 0 )
 	{
@@ -569,7 +624,8 @@ void DVgrab::startCapture()
 		nanosleep( &t, NULL );
 	}
 
-	if ( m_frame )
+	// OK, we have data, commence capture
+	if ( !g_done && m_frame )
 	{
 		sendEvent( "Capture Started" );
 		m_captureActive = true;
@@ -582,7 +638,9 @@ void DVgrab::startCapture()
 		if ( m_dst_file_name )
 			pthread_mutex_unlock( &capture_mutex );
 	}
-	else
+	
+	// No data received in N seconds, throw an error
+	else if ( !g_done )
 	{
 		if ( m_dst_file_name )
 			pthread_mutex_unlock( &capture_mutex );
@@ -591,6 +649,11 @@ void DVgrab::startCapture()
 			reset_bus( m_port );
 		throw std::string( err );
 	}
+	else
+	{
+		if ( m_dst_file_name )
+			pthread_mutex_unlock( &capture_mutex );
+	}
 }
 
 
@@ -727,7 +790,7 @@ void *DVgrab::captureThread( void *arg )
 
 void DVgrab::sendCaptureStatus( const char *name, float size, int frames, TimeCode *tc, struct tm *rd, bool newline )
 {
-	char tc_str[32], rd_str[32];
+	char tc_str[64], rd_str[128];
 
 	if ( tc )
 		sprintf( tc_str, "%2.2d:%2.2d:%2.2d.%2.2d", 
@@ -774,7 +837,7 @@ void DVgrab::writeFrame()
 			timeCode = 0;
 		if ( !m_frame->GetRecordingDate( rd ) )
 		{
-			// If the month is invalid, then report system date/time
+			// If the month is invalid, then report system date/timem_reader_active
 			time_t timesys;
 			time( &timesys );
 			localtime_r( &timesys, recDate );
@@ -820,6 +883,8 @@ void DVgrab::writeFrame()
 			throw std::string( "writing failed" );
 		}
 
+		m_isNewFile |= m_writer->IsNewFile();
+
 		if ( m_writer->IsNewFile() && !m_writer->IsFirstFile() )
 		{
 			sendCaptureStatus( fileName.c_str(), size, framesWritten, lasttc, lastrd, true );
@@ -843,6 +908,17 @@ void DVgrab::writeFrame()
 		{
 			memcpy( &m_lastRecDate, recDate, sizeof( m_lastRecDate ) );
 			m_isLastRecDateSet = true;
+			if ( m_srt )
+			{
+				if ( m_isNewFile )
+				{
+					subWriter.newFile( m_writer->GetFileName().c_str() );
+					m_isNewFile = false;
+				}
+				if ( !subWriter.hasFrameRate() )
+					subWriter.setFrameRate( m_frame->GetFrameRate() );
+				subWriter.addRecordingDate( *recDate, tc );
+			}
 		}
 	}
 	pthread_mutex_unlock( &capture_mutex );
@@ -958,7 +1034,15 @@ void DVgrab::captureThreadRun()
 				TimeCode timeCode = { 0, 0, 0, 0 };
 				dvframe->GetTimeCode( timeCode );
 				if ( dvframe->IsNormalSpeed() &&
-				     ( m_jpeg_overwrite || !( timeCode.hour == 0 && timeCode.min == 0  && timeCode.sec == 0 && timeCode.frame == 0 ) ) )
+				     ( m_jpeg_overwrite ||
+				       !m_avc ||
+				       !m_isRecordMode ||
+				       ( m_isRecordMode &&
+				         strcmp( avc1394_vcr_decode_status( m_avc->TransportStatus( m_node ) ), "Recording" ) == 0 &&
+				         !( timeCode.hour == 0 && timeCode.min == 0  && timeCode.sec == 0 && timeCode.frame == 0 )
+				       )
+				     )
+				   )
 					writeFrame();
 			}
 
@@ -1195,6 +1279,13 @@ bool DVgrab::done()
 {
 	if ( m_reader_active )
 	{
+		// Stop capture at end of tape
+		if ( !m_interactive && m_writer && m_writer->GetFileSize() > 0 && m_avc &&
+		     AVC1394_MASK_RESPONSE_OPERAND( m_avc->TransportStatus( m_node ), 3 ) == AVC1394_VCR_OPERAND_WIND_STOP )
+		{
+			return true;
+		}
+		
 		timespec t = {0, 25000000};
 		return ( nanosleep( &t, NULL ) == -1 );
 	}
@@ -1213,10 +1304,9 @@ void DVgrab::cleanup()
 		pthread_join( capture_thread, NULL );
 		delete m_reader;
 	}
-	if ( m_avc )
-		delete m_avc;
-	if ( m_connection )
-		delete m_connection;
-	if ( m_timeDuration )
-		delete m_timeDuration;
+	delete m_avc;
+	delete m_connection;
+	delete m_timeDuration;
+	if (m_dst_file_name)
+		free(m_dst_file_name);
 }
