@@ -108,6 +108,11 @@ private:
 	int m_jvc_p25;
 	int m_24p;
 	int m_24pa;
+	int m_timeSplit;
+	int m_srt;
+	bool m_isNewFile;
+	bool m_isRecordMode;
+	int m_isRewindFirst;
 
 	static FileHandler *m_writer;
 	bool m_captureActive;
