@@ -33,6 +33,7 @@ using std::endl;
 using std::ostringstream;
 using std::setw;
 using std::setfill;
+using std::ends;
 
 #include <stdio.h>
 #include <signal.h>
@@ -42,6 +43,7 @@ using std::setfill;
 #include <assert.h>
 #include <time.h>
 #include <errno.h>
+#include <string.h>
 
 #include "filehandler.h"
 #include "error.h"
@@ -115,7 +117,7 @@ void FileTracker::Clear( )
 }
 
 FileHandler::FileHandler() :
-	done( false ), autoSplit( false ), maxFrameCount( 0 ), isNewFile( false ), isFirstFile( -1 ),
+        done( false ), autoSplit( false ), timeSplit(0), maxFrameCount( 0 ), isNewFile( false ), isFirstFile( -1 ),
 	lastCollectionFreeSpace( 0 ), currentCollectionSize( 0 ), framesWritten( 0 ), filename( "" )
 {
 	prevTimeCode.sec = -1;
@@ -160,6 +162,11 @@ bool FileHandler::GetAutoSplit()
 	return autoSplit;
 }
 
+int FileHandler::GetTimeSplit()
+{
+	return timeSplit;
+}
+
 
 bool FileHandler::GetTimeStamp()
 {
@@ -223,6 +230,12 @@ void FileHandler::SetAutoSplit( bool flag )
 }
 
 
+void FileHandler::SetTimeSplit( int secs )
+{
+	timeSplit = secs;
+}
+
+
 void FileHandler::SetTimeStamp( bool flag )
 {
 	timeStamp = flag;
@@ -337,11 +350,25 @@ bool FileHandler::WriteFrame( Frame *frame )
 	int time_diff = 0;
 	if ( frame->GetTimeCode( tc ) )
 		time_diff = TIMECODE_TO_SEC(tc) - TIMECODE_TO_SEC(prevTimeCode);
-	bool discontinuity = prevTimeCode.sec != -1 && ( time_diff > 1 || ( time_diff < 0 && time_diff > -59 ) );
+	bool discontinuity = prevTimeCode.sec != -1 && ( time_diff > 1 || time_diff < 0 );
+
+	bool isTimeSplit = false;
+	if ( timeSplit != 0 )
+	{
+		struct tm rd;
+		if ( frame->GetRecordingDate( rd ) )
+		{
+			time_t now = mktime( &rd );
+			if ( now >= prevTime + timeSplit )
+				isTimeSplit = true;
+			prevTime = now;
+		}
+	}
 
 	// If the user wants autosplit, start a new file if a new recording is detected
 	// either by explicit frame flag or a timecode discontinuity
-	if ( FileIsOpen() && GetAutoSplit() && ( frame->IsNewRecording() || discontinuity ) )
+	if ( FileIsOpen() && ( ( GetAutoSplit() && ( frame->IsNewRecording() || discontinuity ) )
+		|| isTimeSplit ) )
 	{
 		CollectionCounterUpdate();
 		Close();
@@ -369,14 +396,14 @@ bool FileHandler::WriteFrame( Frame *frame )
 				<< setw( 2 ) << date.tm_mday << '_'
 				<< setw( 2 ) << date.tm_hour << '-'
 				<< setw( 2 ) << date.tm_min << '-'
-				<< setw( 2 ) << date.tm_sec;
+				<< setw( 2 ) << date.tm_sec << ends;
 				recDate = sb.str();
 			}
 			else
 			{
 				goto TimeSys;
 			}
-			sb2 << GetBaseName() << recDate << GetExtension();
+			sb2 << GetBaseName() << recDate << GetExtension() << ends;
 			filename = sb2.str();
 		}
 		else if ( GetTimeSys() )
@@ -397,7 +424,7 @@ TimeSys:
 			<< setw( 2 ) << date.tm_hour << '-'
 			<< setw( 2 ) << date.tm_min << '-'
 			<< setw( 2 ) << date.tm_sec
-			<< GetExtension();
+			<< GetExtension() << ends;
 			filename = sb.str();
 		}
 		else if ( GetTimeCode() )
@@ -421,6 +448,7 @@ TimeSys:
 				<< "EE:EE:EE:EE"
 				<< GetExtension();
 			}
+			sb << ends;
 			filename = sb.str();
 		}
 		else
@@ -429,7 +457,8 @@ TimeSys:
 			do
 			{
 				ostringstream sb;
-				sb << GetBaseName() << setfill( '0' ) << setw( 3 ) << ++ counter << GetExtension();
+				sb << GetBaseName() << setfill( '0' ) << setw( 3 ) << ++ counter
+					<< GetExtension() << ends;
 				filename = sb.str();
 			}
 			while ( stat( filename.c_str(), &stats ) == 0 );
@@ -552,7 +581,8 @@ int RawHandler::Close()
 off_t RawHandler::GetFileSize()
 {
 	struct stat file_status;
-	fstat( fd, &file_status );
+	if ( fstat( fd, &file_status ) < 0 )
+		return 0;
 	return file_status.st_size;
 }
 
@@ -918,7 +948,7 @@ int QtHandler::Write( Frame *f )
 		TimeCode tc;
 		frame->GetTimeCode( tc );
 
-		if ( tc.frame + frameOffset % 5 )
+		if ( ( tc.frame + frameOffset ) % 5 )
 		{
 			result = quicktime_write_frame( fd, const_cast<unsigned char*>( frame->data ),
 			                                frame->GetFrameSize(), 0 );
@@ -982,8 +1012,8 @@ off_t QtHandler::GetFileSize()
 	if ( fd )
 	{
 		struct stat file_status;
-		stat( filename.c_str(), &file_status );
-		return file_status.st_size;
+		if ( stat( filename.c_str(), &file_status ) == 0 )
+			return file_status.st_size;
 	}
 	else
 		return 0;
@@ -1215,14 +1245,16 @@ int JPEGHandler::Write( Frame *frame )
 		else
 		{
 			ostringstream sb;
-			sb << GetBaseName() << GetExtension();
+			sb << GetBaseName() << GetExtension() << ends;
 			file = sb.str();
 		}
 	}
 	else
 	{
 		ostringstream sb;
-		sb << filename.substr( 0, filename.find_last_of( '.' ) ) << "-" << setfill( '0' ) << setw( 8 ) << ++count << GetExtension();
+		sb << filename.substr( 0, filename.find_last_of( '.' ) ) << "-" 
+			<< setfill( '0' ) << setw( 8 ) << ++count
+			<< GetExtension() << ends;
 		file = sb.str();
 	}
 
@@ -1364,7 +1396,8 @@ int Mpeg2Handler::Close()
 off_t Mpeg2Handler::GetFileSize()
 {
 	struct stat file_status;
-	fstat( fd, &file_status );
+	if ( fstat( fd, &file_status ) < 0 )
+		return 0;
 	return file_status.st_size;
 }
 
