@@ -76,6 +76,7 @@ public:
 	virtual ~FileHandler();
 
 	virtual bool GetAutoSplit();
+	virtual int  GetTimeSplit();
 	virtual bool GetTimeStamp();
 	virtual bool GetTimeSys();
 	virtual bool GetTimeCode();
@@ -91,6 +92,7 @@ public:
 	virtual int GetTotalFrames() = 0;
 
 	virtual void SetAutoSplit( bool );
+	virtual void SetTimeSplit(int secs);
 	virtual void SetTimeStamp( bool );
 	virtual void SetTimeSys( bool );
 	virtual void SetTimeCode( bool );
@@ -143,6 +145,7 @@ protected:
 	bool isNewFile;
 	bool done;
 	bool autoSplit;
+	int  timeSplit;
 	bool timeStamp;
 	bool timeSys;
 	bool timeCode;
@@ -162,6 +165,7 @@ protected:
 	TimeCode prevTimeCode;
 	bool filmRate;
 	bool remove2332;
+	time_t prevTime;
 };
 
 
