@@ -49,6 +49,7 @@
 
 #include <deque>
 #include <iostream>
+#include <typeinfo>
 
 using std::endl;
 
@@ -62,6 +63,7 @@ using std::endl;
 #include <errno.h>
 #include <time.h>
 #include <sys/time.h>
+#include <string.h>
 
 #include <libavc1394/avc1394.h>
 #include <libavc1394/avc1394_vcr.h>
@@ -444,6 +446,11 @@ void iec61883Reader::Close()
 		iec61883_mpeg2_close( m_iec61883_mpeg2 );
 		m_iec61883_mpeg2 = NULL;
 	}
+	if ( m_handle )
+	{
+		raw1394_destroy_handle( m_handle );
+		m_handle = NULL;
+	}
 }
 
 bool iec61883Reader::StartReceive()
@@ -598,8 +605,10 @@ void iec61883Connection::CheckConsistency( int port, int node )
 {
 	raw1394handle_t handle = raw1394_new_handle_on_port( port );
 	if ( handle )
+	{
 		iec61883_cmp_normalize_output( handle, 0xffc0 | node );
-	raw1394_destroy_handle( handle );
+		raw1394_destroy_handle( handle );
+	}
 }
 
 
