@@ -109,6 +109,7 @@ void Time::parseTimeValue( string time )
 			c = time.at( pos );
 			if ( c == '+' || c == '-' )
 			{
+				token << std::ends;
 				string symbol = token.str();
 				token.str( string() );
 				//cerr << "smil: parsed symbol token '" << symbol << "'" << endl;
@@ -150,6 +151,7 @@ void Time::parseTimeValue( string time )
 			}
 			else if ( c == '.' && ( pos == 0 || time.at( pos - 1 ) != '\\' ) )
 			{
+				token << std::ends;
 				base = token.str();
 				token.str( string() );
 				//cerr << "smil: parsed base token '" << base << "'" << endl;
@@ -159,6 +161,7 @@ void Time::parseTimeValue( string time )
 				token << c;
 			}
 		}
+		token << std::ends;
 		string remaining = token.str();
 		//cerr << "smil: parsed remaining token '" << remaining << "'" << endl;
 		if ( !remaining.empty() )
@@ -345,6 +348,7 @@ string Time::toString( TimeFormat format )
 		default:
 			break;
 	}
+	str << std::ends;
 	return str.str();
 
 }
@@ -599,6 +603,7 @@ string MediaClippingTime::toString( TimeFormat format )
 				str << ".0";
 			else if ( m_subframe == SMIL_SUBFRAME_1 )
 				str << ".1";
+			str << std::ends;
 
 			return str.str();
 		}
@@ -689,7 +694,7 @@ string MediaClippingTime::parseFramesToString( int frames, TimeFormat format )
 		case TIME_FORMAT_FRAMES:
 		{
 			std::ostringstream str;
-			str << frames;
+			str << frames << std::ends;
 			return str.str();
 		}
 		
